import { getDevOpsCore } from '@thatdevops/core/devopsCore'
import echo from '@thatdevops/core/utils/echo'
import lerna from '@thatdevops/core/utils/lerna'
import { makeRunnable } from '@thatdevops/core/utils/makeRunable'
import spawn from '@thatdevops/core/utils/spawn'
import doNothing from '@thats/utils/misc/doNothing'
import doResyncPackages from './_/doResyncPackages'

export const gitCommitBumpPublish = makeRunnable<{
	commitMessage?: string
	deploy?: boolean
	force?: boolean
}>(async o => {
	const { publishable } = getDevOpsCore()
	if (!publishable) {
		echo('Publishing not allowed (set the dev ops "publishable" flag')
		return
	}

	// await codeArtifactLogin()

	echo('Resyncing Package versions')
	await doResyncPackages(false)

	const premessage = 'Prepublish - ' + o.commitMessage + ' [skip-cd]'
	echo(`Committing "${premessage}"`)
	await spawn('git', ['add', '-A']).catch(doNothing)
	await spawn('git', ['commit', '-a', '-m', premessage]).catch(doNothing)

	echo('Running Lerna Publish')
	let params = ['patch', '--yes', '--no-push', '--exact']
	if (o.force) {
		params.push('--force-publish')
	}
	echo(`lerna publish ${params.join(' ')}`)
	await lerna.publish(params)

	const postmessage = 'Postpublish - ' + o.commitMessage + (o.deploy ? '' : ' [skip-cd]')
	echo(`Committing "${postmessage}"`)
	await spawn('git', ['commit', '-a', '-m', postmessage]).catch(doNothing)

	echo(`Pushing to git`)
	await spawn('git', ['push']).catch(doNothing)
})
export default gitCommitBumpPublish
